﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Reflection;

namespace Tip43.MvcValidation
{
    public class Validation
    {
        public static void ValidateAttributes<TEntity>(TEntity entity)
        {
            var validationInstance = new Validation();
            validationInstance.ValidateAttributesInternal(entity);
        }

        
        public virtual void ValidateAttributesInternal<TEntity>(TEntity entity)
        {
            var validationIssues = new List<ValidationIssue>();

            // Get list of properties from validationModel
            var props = typeof(TEntity).GetProperties();

            // Perform validation on each property
            foreach (var prop in props)
                ValidateProperty(validationIssues, entity, prop);

            // throw exception?
            if (validationIssues.Count > 0)
                throw new ValidationIssueException(validationIssues);
        }



        protected virtual void ValidateProperty<TEntity>(List<ValidationIssue> validationIssues, TEntity entity, PropertyInfo property)
        {
            // Get list of validator attributes
            var validators = property.GetCustomAttributes(typeof(ValidationAttribute), true);
            foreach (ValidationAttribute validator in validators)
                ValidateValidator(validationIssues, entity, property, validator);
        }

        protected virtual void ValidateValidator<TEntity>(List<ValidationIssue> validationIssues, TEntity entity, PropertyInfo property, ValidationAttribute validator)
        {
            var value = property.GetValue(entity, null);
            if (!validator.IsValid(value))
            {
                validationIssues.Add(new ValidationIssue(property.Name, value, validator.ErrorMessage));
            }
        }



    }
}
